<?php

/*
 * Скрипт защиты от CSRF
 * Автор - rekmixa
 */

// Имя токена
define('CSRF_TOKEN_NAME', 'csrf_token');
// Размер токена (в байтах)
define('CSRF_TOKEN_LENGTH', 32);

/* Генерация токена */

if (!isset($_SESSION[CSRF_TOKEN_NAME])) {

    $php5 = version_compare(PHP_VERSION, '5.3.0');
    $php7 = version_compare(PHP_VERSION, '7.0.0');

    if ($php7 < 0 && $php5 >= 0) {

        // >= PHP 5.3.0
        $bytes = openssl_random_pseudo_bytes(CSRF_TOKEN_LENGTH);

    } elseif ($php7 >= 0) {

        // >= PHP 7.0.0
        $bytes = random_bytes(CSRF_TOKEN_LENGTH);

    }

    $_SESSION[CSRF_TOKEN_NAME] = bin2hex($bytes);

}

/* Проверка токена */

if ($_POST) {

    if (isset($_POST[CSRF_TOKEN_NAME]) && isset($_SESSION[CSRF_TOKEN_NAME])) {

        if ($_POST[CSRF_TOKEN_NAME] != $_SESSION[CSRF_TOKEN_NAME]) {

            // Сбрасываем токен
            unset($_SESSION[CSRF_TOKEN_NAME]);
            // Удаляем зловредные данные
            unset($_POST);

        }

    } else {

        // удаляем безбилетника :)
        unset($_POST);

    }

}

/* Добавляем скрытые поля с токеном во ВСЕ формы */

function csrf_inputs($buffer) {

    $doc = new DOMDocument;

    $doc->loadHTML($buffer);

    $forms = $doc->getElementsByTagName('form'); // ищем формы

    foreach ($forms as $element) {

        // Узнаем метод передачи данных с формы
        $method = $element->getAttribute('method');
        $method = strtoupper($method);

        // Работаем только с POST-данными
        if ($method != 'POST')
            continue;

        /* Создаем поле */

        $newElement = $doc->createElement('input');

        $node = $element->appendChild($newElement);

        $node->setAttribute('type', 'hidden');
        $node->setAttribute('name', CSRF_TOKEN_NAME);
        $node->setAttribute('value', $_SESSION[CSRF_TOKEN_NAME]);

    }

    return $doc->saveHTML();

}

// Буферизация вывода
ob_start('csrf_inputs');

 ?>